@@ -186,6 +186,10 @@ void program_build_instance::serialize_program(void) throw(int)
         src_hw_info[0]='S';
         src_hw_info[1]='K';
         src_hw_info[2]='L';
+    }else if(IS_BROXTON(gen_pci_id)){
+        src_hw_info[0]='B';
+        src_hw_info[1]='X';
+        src_hw_info[2]='T';
     }
 
     if (str_fmt_out) {
